TEMPLATE_SOURCE_TYPE = "TemplateSourceType"

# stairlight.yaml
STAIRLIGHT_CONFIG_FILE_PREFIX = "stairlight"
STAIRLIGHT_CONFIG_INCLUDE_SECTION = "Include"
STAIRLIGHT_CONFIG_EXCLUDE_SECTION = "Exclude"
STAIRLIGHT_CONFIG_SETTING_SECTION = "Settings"

DEFAULT_TABLE_PREFIX = "DefaultTablePrefix"
FILE_SYSTEM_PATH = "FileSystemPath"
REGEX = "Regex"

# mapping.yaml
MAPPING_CONFIG_FILE_PREFIX = "mapping"
MAPPING_CONFIG_GLOBAL_SECTION = "Global"
MAPPING_CONFIG_MAPPING_SECTION = "Mapping"
MAPPING_CONFIG_METADATA_SECTION = "Metadata"

FILE_SUFFIX = "FileSuffix"
LABELS = "Labels"
MAPPING_PREFIX = "MappingPrefix"
PARAMETERS = "Parameters"
TABLE_NAME = "TableName"
TABLES = "Tables"

# GCS
BUCKET_NAME = "BucketName"
PROJECT_ID = "ProjectId"
URI = "Uri"

# Redash
DATABASE_URL_ENVIRONMENT_VARIABLE = "DatabaseUrlEnvironmentVariable"
DATA_SOURCE_NAME = "DataSourceName"
QUERY_IDS = "QueryIds"
QUERY_ID = "QueryId"
