TABLE_NAME = "TableName"
TEMPLATE_SOURCE_TYPE = "TemplateSourceType"
KEY = "Key"
URI = "Uri"
LINES = "Lines"
LINE_NUMBER = "LineNumber"
LINE_STRING = "LineString"
BUCKET_NAME = "BucketName"
LABELS = "Labels"
DATA_SOURCE_NAME = "DataSourceName"

TEMPLATE = "Template"
PARAMETERS = "Parameters"
